#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: pdfutil.py
# Date: Mon Jan 04 02:39:11 2016 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from ..lib.textutil import filter_nonascii, parse_file_size
from ..lib.ukutil import check_file_type, check_buf_filetype
from ..uklogger import *
from .. import ukconfig

import tempfile
import os

def check_buf_pdf(buf):
    return check_buf_filetype(buf, 'PDF document')

def check_legal_pdf(buf):
    def is_exe(path):
        return os.path.isfile(path) and os.access(path, os.X_OK)
    for d in os.environ["PATH"].split(os.pathsep):
        d = d.strip('"')
        exe = os.path.join(d, 'pdftk')
        if is_exe(exe):
            break
    else:
        log_info('pdftk not installed. Not sure if pdf file is valid!')
        return True
    f = tempfile.NamedTemporaryFile(delete=False)
    f.write(buf)
    f.close()
    ret = os.system('pdftk "{}" output /dev/null'.format(f.name))
    os.unlink(f.name)
    return ret == 0

def pdf2text(data):
    f = tempfile.NamedTemporaryFile(delete=False, suffix='.pdf')
    f.write(data)
    f.close()

    #with timeout(seconds=30):
    ret = os.system('pdftotext "{0}"'.format(f.name))
        #ret = timeout_command('pdftotext "{0}"'.format(os.path.realpath(f.name)),
                             #3)
    if ret != 0:
        #raise Exception("Timeout in pdf2text")
        raise Exception("pdftotext return error! original file: {0}".format(f.name))
    fout = f.name.replace('.pdf', '.txt')
    text = open(fout).read()

    os.remove(f.name)
    os.remove(fout)

    text = filter_nonascii(text)
    # TODO filter formulas..
    return text

def pdf_compress(data):
    """ take a pdf data string, return a compressed string
        compression is done using ps2pdf14 in ghostscript
    """
    f = tempfile.NamedTemporaryFile(delete=False, suffix='.pdf')
    f.write(data)
    f.close()

    f2 = tempfile.NamedTemporaryFile(delete=False, suffix='.pdf')
    f2.close()
    ret = os.system('ps2pdf14 "{0}" "{1}"'.format(f.name, f2.name))
    if ret != 0:
        log_err("Compress: ps2pdf14 failed!")
        newdata = None
    else:
        newdata = open(f2.name).read()
    file_succ = newdata is not None and \
            check_file_type(f2.name, 'PDF document') and \
            len(newdata) >= ukconfig.FILE_SIZE_MINIMUM
    try:
        os.remove(f2.name)
        os.remove(f.name)
    except OSError:
        pass
    if file_succ and \
       len(newdata) < len(data):
        log_info("Compress succeed: {0}->{1}".format(
            parse_file_size(len(data)), parse_file_size(len(newdata))))
        return newdata
    else:
        return data



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: xpcommon.py
# Date: Fri May 23 12:35:53 2014 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

FIELD_NUM = {'id': 0,
             'title': 1,
             'text': 2,
             'author': 3
            }

import os

STOPWORDS_FILE = os.path.join(os.path.dirname(__file__), 'stopwords.txt')

STOPWORDS = set([x.strip() for x in open(STOPWORDS_FILE).readlines()])



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: contentsearch.py
# Date: 五 6月 13 16:55:19 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import tempfile
import os
from threading import Lock, Condition
from lib.timeout import timeout, timeout_command

from xpengine.indexer import XapianIndexer
from xpengine.searcher import XapianSearcher
from lib.singleton import Singleton
import ukconfig
from ukdbconn import get_mongo
from uklogger import *
from lib.textutil import filter_nonascii
from lib.pdfutil import pdf2text

DB_DIR = ukconfig.XP_DB_DIR


class SoPaperSearcher(object):
    """ Search by content of paper
        Don't instantiate me
    """
    __metaclass__ = Singleton

    def __init__(self):
        if not os.path.isdir(DB_DIR):
            SoPaperIndexer().rebuild()
        self.searcher = XapianSearcher(DB_DIR)

    def search(self, query, offset=0,
               page_size=ukconfig.SEARCH_PAGE_SIZE,
               summary_len=ukconfig.SEARCH_SUMMARY_LEN):
        res = self.searcher.search(query, offset, page_size, summary_len)
        return res

class SoPaperIndexer(object):
    """ Don't instantiate me
    """
    __metaclass__ = Singleton

    def __init__(self):
        self.indexer = XapianIndexer(DB_DIR)

    def _do_add_paper(self, doc):
        try:
            self.indexer.add_doc(doc)
        except:
            log_exc("Exception in add_paper")
            log_info("Error with this doc: {0}".format(doc['id']))

    def add_paper(self, doc):
        assert doc.get('text')
        assert doc.get('title')
        assert doc.get('id')
        self._do_add_paper(doc)
        self.indexer.flush()
        SoPaperSearcher().searcher.reopen()

    def rebuild(self):
        self.indexer.clear()

        db = get_mongo('paper')
        itr = db.find({}, {'pdf': 1, 'title': 1, 'text': 1})
        for res in itr:
            text = res.get('text')
            if not text:
                log_info("About to add text for paper {0}".format(res['_id']))
                try:
                    data = res['pdf']
                    text = pdf2text(data)
                except KeyError:
                    log_err("No pdf in pid={0},title={1}".format(
                        res['_id'], res['title']))
                    continue
                except Exception:
                    log_exc("Exception in pdf2text")

                db.update({'_id': res['_id']}, {'$set': {'text': text}})
            doc = {'text': text,
                   'title': res['title'],
                   'id': res['_id']
                  }
            self._do_add_paper(doc)
        self.indexer.flush()

indexer_lock = Lock()
def do_add_paper(doc):
    indexer_lock.acquire()
    idxer = SoPaperIndexer()
    idxer.add_paper(doc)
    SoPaperSearcher().searcher.reopen()
    indexer_lock.release()

if __name__ == '__main__':
    print "Rebuilding..."
    SoPaperIndexer().rebuild()



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: ukconfig.py
# Date: Mon Jan 04 02:33:47 2016 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

download_method = 'wget'
USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.152 Safari/537.36'

FILE_SIZE_MINIMUM = 10000        # at least 10kb
FILE_SIZE_MAXIMUM = 100000000    # at most 100mb

PYTHON_POOL_TIMEOUT = 9999  # bug in python: see http://stackoverflow.com/questions/1408356/keyboard-interrupts-with-pythons-multiprocessing-pool

import os

LOG_DIR = None

# this lib is not required for command line script
USE_MAGIC_LIB = False
try:
    import magic
except ImportError:
    USE_MAGIC_LIB = False

USE_DB = True
USE_INDEXER = True
try:
    import pymongo
except ImportError:
    USE_DB = False

mongo_conn = ('127.0.0.1', 27018)
mongo_db = 'sopaper'

BS_PARSER = 'html.parser' # parser for beautifulsoup

DB_DIR_NAME = 'xapian-db'
XP_DB_DIR = os.path.join(os.path.dirname(__file__),
                         '../{0}'.format(DB_DIR_NAME))

SEARCH_PAGE_SIZE = 10
SEARCH_SUMMARY_LEN = 300

MAILSERVER_HOST = 'server4.net9.org'
MAILSERVER_PORT = 5184



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: dbsearch.py
# Date: 六 6月 14 03:18:57 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import operator

from ukdbconn import get_mongo
from uklogger import *
from lib.textutil import title_beautify, levenshtein

SEARCH_RETURN_FIELDS = {'view_cnt': 1, 'download_cnt': 1,
                        'title': 1, 'page': 1, 'source': 1,
                        'page_url': 1,
                        'author': 1, 'bibtex': 1, 'abstract': 1,
                        'references': 1, 'citedby': 1,
                        'comments': {'$slice': 10},
                        'cmt_count': 1}

def beautify_results():
    def wrap(func):
        def call(query):
            res = func(query.lower())
            for k in res:
                k['title'] = title_beautify(k['title'])
            return res
        return call
    return wrap

@beautify_results()
def search_exact(query):
    db = get_mongo('paper')
    res = list(db.find({'title': query}, SEARCH_RETURN_FIELDS))
    return res

@beautify_results()
def search_startswith(query):
    db = get_mongo('paper')
    res = list(db.find({'title':
                        {'$regex': '^{0}'.format(query) } },
                       SEARCH_RETURN_FIELDS))
    res = [k for k in res if levenshtein(k['title'], query) < 10]
    print res
    return res

@beautify_results()
def search_regex(regex):
    db = get_mongo('paper')
    res = list(db.find({'title': {'$regex':
                                  '{0}'.format(query) }
                       }, SEARCH_RETURN_FIELDS))
    return res


# XXX Hack!!
# Similar Search in cached memory

all_titles = []
def similar_search(query):
    """ return one result that is most similar to query"""
    ret = []
    query = query.strip().lower()
    for cand in all_titles:
        dist = levenshtein(query, cand[0])
        if dist < 3:
            ret.append((cand, dist))
    if not ret:
        return None
    res = max(ret, key=operator.itemgetter(1))

    db = get_mongo('paper')
    res = db.find_one({'_id': res[0][1]}, SEARCH_RETURN_FIELDS)
    return res


def add_title_for_similar_search(cand):
    """ cand = (title, id) """
    all_titles.append((cand[0].strip().lower(), cand[1]))

def init_title_for_similar_search():
    if len(all_titles) > 0:
        return
    db = get_mongo('paper')
    itr = db.find({}, {'title': 1})
    for cand in itr:
        add_title_for_similar_search((cand['title'], cand['_id']))

init_title_for_similar_search()

if __name__ == '__main__':
    print search_exact(title_beautify('Intriguing properties of neural networks'))



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: test-searcher.py
# Date: Tue Jan 20 14:22:43 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from multiprocessing import Pool
import sys

import searcher
from job import JobContext
from searcher import searcher_run

if __name__ == '__main__':
    query = sys.argv[1]
    searchers = searcher.register_searcher.get_searcher_list()
    searchers = searchers[1:]
    print [k.name for k in searchers]
    ctx = JobContext(query)

    args = zip(searchers, [ctx] * len(searchers))
    pool = Pool()
    async_results = [pool.apply_async(searcher_run, arg) for arg in args]

    # Search and get all the results item
    for s in async_results:
        s = s.get()
        if s is None:
            continue
        srs = s['results']

        print srs

        meta = s.get('ctx_update')
        if meta:
            ctx.update_meta_dict(meta)
    pool.close()
    pool.terminate()



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: sciencedirect.py
# Date: 一 6月 09 17:06:26 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import re
from . import register_parser, RecoverableErr
from .base import FetcherBase, direct_download
from ..uklogger import *
from .. import ukconfig

from urlparse import urlparse
import requests
from bs4 import BeautifulSoup

HOSTNAME = 'www.sciencedirect.com'

# not working right now
#@register_parser(name='sciencedirect.com', urlmatch='sciencedirect.com',
                 #meta_field=['author', 'bibtex', 'abstract'],
                 #priority=8)
class ScienceDirect(FetcherBase):
    def _do_pre_parse(self):
        self.text = requests.get(self.url).text.encode('utf-8')
        with open("/tmp/b.html", 'w') as f:
            f.write(self.text)
        #text = open("/tmp/b.html").read()
        self.soup = BeautifulSoup(self.text)

    def _do_download(self, updater):
        pdf = self.soup.findAll(attrs={'id': 'pdfLink'})
        if pdf:
            try:
                url = pdf[0]['pdfurl']
                print url
            except:
                # probably something need to be fixed
                log_exc('')
        else:
            raise RecoverableErr("No available download at {0}".format(self.url))
        return direct_download(url, updater)

    def _do_get_title(self):
        titles = self.soup.findAll(attrs={'name': 'citation_title'})
        return titles[0]['content']

    def _do_get_meta(self):
        meta = {}
        try:
            log_info("Getting author...")
            authors = self.soup.findAll(
                attrs={'title': 'Author Profile Page'})
            author = [a.text for a in authors]
            meta['author'] = author
        except KeyboardInterrupt:
            raise
        except:
            pass

        try:
            log_info("Getting abstract...")
            abstract_url = re.findall(r'\'tab_abstract.+\d+\'', self.text)[0][1:-1]
            abstract_text = requests.get('http://{0}/'.format(HOSTNAME) + abstract_url).text.encode('utf-8')
            abstract_soup = BeautifulSoup(abstract_text)
            abstract = abstract_soup.findAll('p')[0].text
            meta['abstract'] = abstract
        except KeyboardInterrupt:
            raise
        except:
            pass

        try:
            log_info("Getting refs ...")
            ref_url = re.findall(r'\'tab_references.+\d+\'', self.text)[0][1:-1]
            ref_text = requests.get('http://{0}/'.format(HOSTNAME) + ref_url).text.encode('utf-8')
            ref_soup = BeautifulSoup(ref_text)
            trs = ref_soup.findAll('tr')
            reference = []
            for tr in trs:
                records = tr.findAll('a')
                if len(records) > 0:
                    href = 'http://{0}/'.format(HOSTNAME) + records[0].get('href')
                    ref = records[0].text.strip()
                    reference.append({'ref': ref, 'href': href})
            meta['references'] = reference
        except KeyboardInterrupt:
            raise
        except:
            pass

        try:
            log_info("Getting cited ...")
            cite_url = re.findall(r'\'tab_citings.+\d+\'', self.text)[0][1:-1]
            cite_text = requests.get('http://{0}/'.format(HOSTNAME) +
                                     cite_url, timeout=5
                                    ).text.encode('utf-8')
            cite_soup = BeautifulSoup(cite_text)
            trs = cite_soup.findAll('tr')
            citing = []
            for tr in trs:
                records = tr.findAll('a')
                if len(records) > 0:
                    href = 'http://{0}/'.format(HOSTNAME) + records[0].get('href')
                    cite = records[0].text.strip()
                    citing.append({'citing': cite, 'href': href})
            meta['citedby'] = citing
        except KeyboardInterrupt:
            raise
        except requests.exceptions.Timeout:
            pass
        except:
            pass

        try:
            log_info("Getting bibtex...")
            bibtex_url = re.findall(r'exportformats.+bibtex', self.text)[0]
            bibtex_text = requests.get('http://{0}/'.format(HOSTNAME) + bibtex_url).text.encode('utf-8')
            bibtex_soup = BeautifulSoup(bibtex_text)
            pre = bibtex_soup.find('pre')
            bibtex = pre.text.strip()
            meta['bibtex'] = bibtex
        except KeyboardInterrupt:
            raise
        except:
            pass
        return meta



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: timeout.py
# Date: 二 6月 10 03:44:44 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import signal
class timeout:
    def __init__(self, seconds=1, error_message='Timeout'):
        self.seconds = seconds
        self.error_message = error_message
    def handle_timeout(self, signum, frame):
        raise TimeoutError(self.error_message)
    def __enter__(self):
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)
    def __exit__(self, type, value, traceback):
        signal.alarm(0)

def timeout_command(command, timeout):
    """call shell-command and either return its output or kill it
    if it doesn't normally exit within timeout seconds and return None"""
    import subprocess, datetime, os, time, signal
    start = datetime.datetime.now()
    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    while process.poll() is None:
        time.sleep(0.1)
        now = datetime.datetime.now()
        if (now - start).seconds > timeout:
            os.kill(process.pid, signal.SIGKILL)
            os.waitpid(-1, os.WNOHANG)
            return False
    return True



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: pdf2html.py
# Date: Mon May 26 15:43:21 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import os
import os.path
import tempfile
import shutil

class PDF2Html(object):
    def __init__(self, data, filename):
        """ provide either data or filename"""
        if data is not None:
            f = tempfile.NamedTemporaryFile(delete=False, suffix='.pdf')
            f.close()
            with open(f.name, 'w') as fpdf:
                fpdf.write(data)
            self.fname = f.name
            self.createfile = True
        else:
            self.fname = filename
            self.createfile = False
        assert self.fname is not None
        self.convert()

    def convert(self):
        self.outdir = tempfile.mkdtemp(prefix='sop')
        ret = os.system('pdf2htmlEX "{0}" 0.html --dest-dir={1} --zoom=1.5'.
                  format(self.fname, self.outdir) + \
                  ' --split-pages=1 --page-filename %d.html')
        if ret != 0:
            raise Exception("pdf2htmlEx return error! original file: {0}".format(self.fname))
        self.npages = len(os.listdir(self.outdir)) - 1

    def clean(self):
        shutil.rmtree(self.outdir)
        if self.createfile:
            os.remove(self.fname)

    def get_npages(self):
        return self.npages

    def get(self, t):
        t = int(t)
        assert t >= 0 and t <= self.npages
        fname = os.path.join(self.outdir, '{0}.html'.format(t))
        return open(fname).read()

if __name__ == '__main__':
    w = PDF2Html(data=None, filename='/tmp/a.pdf')
    print len(w.get(1))
    w.clean()






#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: __main__.py
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

# Command line script to use paper-downloader
# You don't need to setup virtualenv to use script
# But you'll need requests and BeautifulSoup4 installed

import sys
import os
import re
import os.path
import argparse
from multiprocessing import Pool

# Config must be set at the beginning
from sopaper import ukconfig
ukconfig.download_method = 'wget'
ukconfig.USE_DB = False
ukconfig.LOG_DIR = None

from sopaper import searcher
from sopaper.searcher import searcher_run
from sopaper.job import JobContext, SearchResult
from sopaper import fetcher
from sopaper.lib.pdfutil import pdf_compress
from sopaper.lib.textutil import finalize_filename, md5
from sopaper.uklogger import *

def get_args():
    desc = 'SoPaper command line tool -- ' \
        'Fully Automated Paper Searcher & Downloader' \
        '\nSoPaper, So Easy'
    parser = argparse.ArgumentParser(description = desc)

    parser.add_argument('title', nargs='+', help='Title of the paper or URL of an arxiv/ieee/dlacm page')
    parser.add_argument('-u', '--url', action='store_true', help='do not download, print URL only')
    parser.add_argument('-d', '--directory',
                        help='Output Directory (default: current directory)',
                        required=False, default='.')
    parser.add_argument('-o', '--output',
                        help='Manually specify a output file, rather than automatically determine the correct name.')
    ret = parser.parse_args()
    ret.title = ' '.join(ret.title)
    return ret

def main():
    global args
    args = get_args()
    query = args.title
    directory = args.directory


    searchers = searcher.register_searcher.get_searcher_list()
    parsers = fetcher.register_parser.get_parser_list()
    download_candidates = []
    if re.match('^http[s]?://', query):
        # skip search
        ctx = JobContext("")
        sr = SearchResult(None, query)
        for parser in parsers:
            if parser.can_handle(sr):
                parser.fetch_info(ctx, sr)      # will update title
                download_candidates.append((parser, sr))
    else:
        #query = "Distinctive image features from scale-invariant keypoint"
        ctx = JobContext(query)

        search_args = zip(searchers, [ctx] * len(searchers))
        pool = Pool()
        as_results = [pool.apply_async(searcher_run, arg) for arg in search_args]
        #results = [searcher_run(*arg) for arg in search_args]  # for debug

        url_seen = set()    # avoid repeated url
        for s in as_results:
            s = s.get(ukconfig.PYTHON_POOL_TIMEOUT)
            if s is None:
                continue
            ctx.update_meta_dict(s['ctx_update'])
            ctx.try_update_title_from_search_result(s)

            for sr in s['results']:
                if sr.url in url_seen:
                    continue
                url_seen.add(sr.url)
                for parser in parsers:
                    if parser.can_handle(sr):
                        parser.fetch_info(ctx, sr)      # will update title
                        download_candidates.append((parser, sr))
        pool.terminate()

    download_candidates = sorted(
        download_candidates,
        key=lambda x: x[0].priority,
        reverse=True)

    if ctx.title:
        ctx.title = finalize_filename(ctx.title)
    else:
        log_info("Failed to guess paper title!")
        ctx.title = "Unnamed Paper {}".format(md5(data))
    if args.url:
        # url mode
        print("Results for {}:".format(ctx.title))
        for (_, sr) in download_candidates:
            print(sr.url)
        return

    for (parser, sr) in download_candidates:
        data = parser.download(sr)
        if not data:
            continue
        data = pdf_compress(data)

        filename = os.path.join(directory, ctx.title + ".pdf")
        if os.path.exists(filename):
            log_err("File \"{}\" exists! overwrite? (y/n)".format(os.path.basename(filename)))
            resp = raw_input()
            if resp not in ['y', 'Y']:
                log_info("No file written. Exiting...")
                break
        with open(filename, 'wb') as f:
            f.write(data)
        if args.output:
            os.rename(filename, args.output)
        break
    else:
        log_err("Failed to download {0}".format(ctx.title))
        return
    if ctx.meta.get('bibtex'):
        log_info("Bibtex:\n{}".format(ctx.meta['bibtex']))
    if ctx.meta.get('author'):
        log_info("Author: {0}".format(ctx.meta['author']))
    if ctx.meta.get('citecnt'):
        log_info("Cite count: {0}".format(ctx.meta['citecnt']))
    log_info("Successfully downloaded to {0}".format(filename))

if __name__ == '__main__':
    main()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# File: ukdbconn.py
# Date: 二 6月 10 04:01:51 2014 +0000
# Author: jiakai <jia.kai66@gmail.com>
#         Yuxin Wu <ppwwyyxxc@gmail.com>

"""database connections"""


try:
    from pymongo import MongoClient
except ImportError:
    from pymongo import Connection as MongoClient
from pymongo.errors import DuplicateKeyError

import ukconfig
from uklogger import *

_db = None

def get_mongo(coll_name=None):
    global _db
    if _db is None:
        _db = MongoClient(*ukconfig.mongo_conn)[ukconfig.mongo_db]

    if coll_name is None:
        return _db
    return _db[coll_name]

def new_paper(ctx):
    pid = global_counter('paper')
    log_info("Add new paper: {0}, pid={1}".format(
        ctx.title, pid))
    doc = {
        '_id': pid,
        'title': ctx.title.lower(),
        'view_cnt': 1,
        'download_cnt': 0
    }
    meta = ctx.meta
    if 'author' in meta:
        meta['author'] = [x.lower() for x in meta['author']]
    doc.update(ctx.meta)
    doc['title'] = doc['title'].lower()

    db = get_mongo('paper')
    db.ensure_index('title')
    ret = db.insert(doc)
    return pid

def update_meta(pid, meta):
    db = get_mongo('paper')
    db.update({'_id': pid}, {'$set': meta})

def update_view_cnt(pid):
    db = get_mongo('paper')
    db.update({'_id': pid}, {'$inc': {'view_cnt': 1}})

def global_counter(name, delta=1):
    """ atomically change a global int64 counter and return the newest value;
    starting from 1
    mongo document structure:
    {
        _id: counter name
        val: current value
    }"""
    db = get_mongo('global_counter')
    rst = db.find_and_modify(query={'_id': name},
                            update={'$inc': {'val': delta}},
                            new=True)
    if rst:
        k = rst.get('val')
        if k:
            return k
    try:
        val = long(1)
        db.insert({'_id': name, 'val': val})
        return val
    except DuplicateKeyError:
        return global_counter(name, delta)



#!../../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: __init__.py
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from ..lib.downloader import ProgressPrinter
from ..lib.textutil import title_beautify
from ..lib.ukutil import import_all_modules, ensure_unicode
from ..lib.pdfutil import check_buf_pdf, check_legal_pdf
from ..uklogger import *
from .. import ukconfig
from ..job import SearchResult
from ..lib.exc import RecoverableErr

if ukconfig.USE_DB:
    import ukdbconn
    from dbsearch import search_exact

from functools import wraps
import re

class register_parser(object):
    parser_dict = {}
    """ save the original parser func"""

    def __init__(self, *args, **kwargs):
        self.name = kwargs.pop('name')
        self.url_match = re.compile(kwargs.pop('urlmatch'))

        """ priority of this parser, higher for those unlikely to be blocked"""
        self.priority = kwargs.pop('priority', 5)

        self.type_match = kwargs.pop('typematch', None)
        self.legal = kwargs.pop('legal', True)

        """ what meta field this fetcher might provide"""
        self.support_meta_field = kwargs.pop('meta_field', [])

        """ whether this fetcher should be considered multiple times l
            during a search.
            For now, only 'DirectPdfParser' shall be considered multiple times
        """
        self.repeatable = kwargs.pop('repeatable', False)

        assert self.name not in self.parser_dict

    @staticmethod
    def get_parser_list():
        lst = register_parser.parser_dict.values()
        return sorted(lst, key=lambda x: x.priority, reverse=True)

    def __call__(self, fetcher_cls):
        """ fetcher_cls: subclass of FetcherBase to be used
        """
        self.fetcher_cls = fetcher_cls

        @wraps(fetcher_cls)
        def wrapper(res):
            assert isinstance(res, SearchResult)
            try:
                fetcher = fetcher_cls(res)
                fetcher.name = self.name
                fetcher.get_title()
                fetcher.get_meta()
                return fetcher
            except KeyboardInterrupt:
                raise
            except Exception:
                log_exc("Error in parser '{0}' with url '{1}'".
                        format(self.name, res.url))
                return None
        self.parser_dict[self.name] = self
        self.cb = wrapper
        return wrapper

    def can_handle(self, sr):
        if (self.type_match is None
            or self.type_match != sr.type) and \
           len(self.url_match.findall(sr.url)) == 0:
            return False
        return True

    def fetch_info(self, ctx, sr):
        """ run this parser against the SearchResult given
            return True/False indicate success,
            will update ctx metadata and ctx.success,

            on success, either ctx will be filled with fetched data,
                        or ctx.existing will contain a existing doc in db.
        """
        url = sr.url
        log_info("Parsing url {0} with parser {1}".
                 format(url, self.name))
        fetcher_inst = self.cb(sr)
        if fetcher_inst is None:
            return False

        newt = fetcher_inst.get_title()
        if newt and ctx.update_new_title(newt):
            # check updated title against db before download
            if ukconfig.USE_DB:
                doc = search_exact(newt)
                if doc:
                    ctx.existing = doc[0]
                    ukdbconn.update_meta(doc[0]['_id'], fetcher_inst.get_meta())
                    return True
        meta = fetcher_inst.get_meta()
        if len(meta):
            log_info("Fetcher {} Update Metadata: {}".format(
                fetcher_inst.name, str(meta.keys())))
        ctx.update_meta_dict(meta)
        return True

    def download(self, sr, progress_updater=None):
        """ return binary data or None"""
        fetcher_inst = self.fetcher_cls(sr)

        succ = fetcher_inst.download(progress_updater)
        if not succ:
            return None

        data = fetcher_inst.get_data()
        ft = check_buf_pdf(data)
        if ft == True:
            ft = check_legal_pdf(data)
            if ft == True:
                return data
            else:
                log_err("Found a broken pdf")
        else:
            log_err("Wrong Format.")


if __name__ != '__main__':
    import_all_modules(__file__, __name__)




#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# $File: uklogger.py
# $Date: 一 6月 09 17:20:34 2014 +0000
# $Author: jiakai <jia.kai66@gmail.com>

"""utilities for handling logging"""

import ukconfig
import traceback
import time
import os
import os.path
from termcolor import colored


def log_api(msg):
    """log a message from api-website"""
    print colored('API', 'green'), msg
    # TODO: use log util, log to file, including time, module, etc.


def log_info(msg):
    """log an info message"""
    print colored('INFO', 'blue'), msg
    if ukconfig.LOG_DIR:
        with open(os.path.join(ukconfig.LOG_DIR, 'info.txt'), 'a') as f:
            f.write(msg)
            f.write('\n')
    # TODO: use log util, log to file, including time, module, etc.


def log_err(msg):
    """log an err message"""
    print colored('ERR', 'red', attrs=['blink']), msg
    if ukconfig.LOG_DIR:
        with open(os.path.join(ukconfig.LOG_DIR, 'error.txt'), 'a') as f:
            f.write(msg)
            f.write('\n')
    # TODO: use log util, log to file, including time, module, etc.

def log_exc(exc):
    """log an unexpected exception"""
    log_err('Caught unexpected exception: {}\n{}'.format(
        exc, traceback.format_exc()))



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: ieee.py
# Date: 五 6月 13 18:22:19 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from . import register_parser
from .base import FetcherBase, direct_download
from ..uklogger import *
from ..ukconfig import BS_PARSER

import re
from bs4 import BeautifulSoup
import requests

HOSTNAME = 'ieeexplore.ieee.org'
STAMP_URL = "http://ieeexplore.ieee.org/stamp/stamp.jsp?tp=&arnumber={0}"
REFERENCE_URL = "http://ieeexplore.ieee.org/xpl/abstractReferences.jsp?tp=&arnumber={0}"
CITATION_URL = "http://ieeexplore.ieee.org/xpl/abstractCitations.jsp?tp=&arnumber={0}"

@register_parser(name='ieeexplore.ieee.org',
                 urlmatch='ieeexplore.ieee.org',
                 meta_field=['author', 'abstract', 'references', 'citedby'])
class IEEE(FetcherBase):
    def _do_pre_parse(self):
        text = requests.get(self.url).text.encode('utf-8')
        self.soup = BeautifulSoup(text, BS_PARSER)

        number = re.findall('arnumber=[0-9]*', self.url)[0]
        self.number = re.findall('[0-9]+', number)[0]

    def _do_download(self, updater):
        url2 = STAMP_URL.format(self.number)
        text = requests.get(url2).text.encode('utf-8')
        soup = BeautifulSoup(text, BS_PARSER)
        fr = soup.findAll('frame')[-1]
        pdflink = fr.get('src')
        return direct_download(pdflink, updater)

    def _do_get_title(self):
        titles = self.soup.findAll('h1')
        title = titles[0].text.strip()
        return title

    def _do_get_meta(self):
        meta = {}
        try:
            authors = self.soup.findAll(attrs={'name': 'citation_author'})
            author = [a.get('content') for a in authors]
            meta['author'] = author
        except KeyboardInterrupt:
            raise
        except:
            pass

        try:
            abstract_div = self.soup.findAll(attrs={'class': 'article'})
            abstract = abstract_div[0].text.strip()
            meta['abstract'] = abstract
        except KeyboardInterrupt:
            raise
        except:
            pass

        try:
            ref_url = REFERENCE_URL.format(self.number)
            ref_text = requests.get(ref_url).text
            ref_soup = BeautifulSoup(ref_text, BS_PARSER)
            ol = ref_soup.findAll('ol')[0]
            lis = ol.findAll('li')
            reference = []
            for li in lis:
                ref = li.text.strip()
                ref = ref.replace('\t', '')
                ref = ref.replace('\n', '')
                ref = ref.replace('\r', '')
                ref = ref.replace('\\', '')
                ref = ref.replace(u'\xa0', u' ')
                if 'Abstract' in ref:
                    ref = ref[:ref.find('Abstract')]
                if 'Cross' in ref:
                    ref = ref[:ref.find('[')]
                href = ''
                link = li.findAll('a')
                if len(link) > 0:
                    href = link[0].get('href')
                    if not 'http' in href:
                        href = "http://{0}{1}".format(HOSTNAME, href)
                reference.append({'ref': ref, 'href': href})
            meta['references'] = reference
        except KeyboardInterrupt:
            raise
        except:
            pass

        try:
            cite_url = CITATION_URL.format(self.number)
            cite_text = requests.get(cite_url).text.encode('utf-8')
            cite_soup = BeautifulSoup(cite_text, BS_PARSER)
            html = cite_soup.findAll('ol')[0]
            lis = html.findAll('li')
            citing = []
            for li in lis:
                cite = li.text.strip()
                cite = cite.replace('\t', '')
                cite = cite.replace('\n', '')
                cite = cite.replace('\r', '')
                cite = cite.replace('\\', '')
                cite = cite.replace('  ', '')
                cite = cite.replace(u'\xa0', u' ')
                if 'Abstract' in cite:
                    cite = cite[:cite.find('Abstract')]
                if 'Cross' in cite:
                    cite = cite[:cite.find('[')]
                href = ''
                link = li.findAll('a')
                if len(link) > 0:
                    href = "http://{0}{1}".format(HOSTNAME,
                        re.findall(r'/xpl/articleDetails.jsp\?arnumber=[0-9]+', link[0].get('href'))[0])
                citing.append({'citing': cite, 'href': href})
            meta['citedby'] = citing
        except KeyboardInterrupt:
            raise
        except:
            pass

        """
        there are still difficulties getting the bibtex
        """

        return meta



#!../../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: textutil.py
# Date: Wed Jul 08 22:33:24 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import string
import re
import os
import hashlib
from .ukutil import ensure_unicode

STOPWORDS = set(['of', 'from', 'as', 'to', 'a', 'an', 'in', 'into', 'on',
                 'onto', 'with', 'about', 'the', 'for', 'and', 'or', 'by',
                 'without', 'instead', 'is', 'are', 'since', 'between',
                 'after', 'befoer', 'then', 'than', 'via'])

ABBR_DICT = [(k, v) for k, v in
            [l.strip().split('\t') for l in open(os.path.join(
                os.path.dirname(__file__), 'abbr.dic')).readlines()]]

def title_beautify(title):
    title = title.strip().lower()
    title = " ".join(title.split())
    tk = title.title().split()
    for (idx, w) in enumerate(tk):
        if w.lower() in STOPWORDS and not idx == 0:
            tk[idx] = w.lower()
        else:
            tk[idx] = w.capitalize()
    return " ".join(tk)

def parse_file_size(size):
    if size > 1048576:
        return "{0:.2f}MB".format(float(size) / 1024 / 1024)
    if size > 1024:
        return "{0:.2f}KB".format(float(size) / 1024)
    return "{0}B".format(size)

def filter_title_fileformat(title):
    title = title.replace('[pdf]', '')
    title = title.replace('[PDF]', '')
    return title

def levenshtein(s1, s2):
    if len(s1) < len(s2):
        return levenshtein(s2, s1)

    if len(s2) == 0:
        return len(s1)

    previous_row = xrange(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            insertions = previous_row[j + 1] + 1 # j+1 instead of j since previous_row and current_row are one character longer
            deletions = current_row[j] + 1       # than s2
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row
    return previous_row[-1]

def title_correct(query, title):
    """ return (match, update) """
    title = title.replace('[PDF]', '')
    q = ''.join([t for t in query if t in string.letters]).lower()
    now = ''.join([t for t in title if t in string.letters]).lower()
    ed_thres = min(len(query), len(title)) / 5
    ERROR_RATIO = 0.6
    if levenshtein(q, now) < ed_thres:
        return (True, True)
    for k in range(min([int(len(query) * ERROR_RATIO), 30]), len(query)):
        if levenshtein(q[:k], now) < ed_thres:
            return (True, False)
    for k in range(int(len(title) * ERROR_RATIO), len(title)):
        if levenshtein(now[:k], q) < ed_thres:
            return (True, False)
    return (False, False)

def name_clean(name):
    p = re.compile('\(.*?\)', re.DOTALL)
    ret = p.sub('', name).strip()
    return ensure_unicode(ret)

def filter_nonascii(string):
    return filter(lambda x: ord(x) < 128, string)

def abbr_subst(s):
    for k, v in ABBR_DICT:
        s = re.sub(k, v, s, flags=re.IGNORECASE)
    return s

def finalize_filename(s):
    s = s.replace('/', ' ')
    s = abbr_subst(s)
    return s

def md5(s):
    m = hashlib.md5()
    m.update(s)
    return m.hexdigest()

if __name__ == '__main__':
    print title_correct("Gated Softmax Classification",
                        "[PDF]Gated Softmax Classification - NIPS Proceedings")



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: queryhandler.py
# Date: Thu Jun 18 22:52:39 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from bson.binary import Binary
from threading import Thread
from multiprocessing import Pool

from ukdbconn import get_mongo, global_counter, new_paper
from uklogger import *
from lib.textutil import title_beautify, parse_file_size
import searcher
from searcher import searcher_run
import fetcher
from job import JobContext
from dbsearch import *
from pdfprocess import postprocess
from lib.downloader import ProgressPrinter
from contentsearch import SoPaperSearcher
import ukconfig

# global. save all ongoing download
progress_dict = {}

class Updater(ProgressPrinter):
    def __init__(self, pid):
        self.pid = pid
        super(Updater, self).__init__()

    def update(self, done):
        percent = float(done) / self.total
        progress_dict[self.pid] = percent
        super(Updater, self).update(done)

def start_download(dl_candidates, ctx, pid):
    dl_candidates = sorted(dl_candidates, key=lambda x: x[0].priority,
                           reverse=True)
    updater = Updater(pid)
    for (parser, sr) in dl_candidates:
        data = parser.download(sr, updater)
        if data:
            db = get_mongo('paper')
            try:
                db.update({'_id': pid},
                          {'$set': {
                            'pdf': Binary(data),
                            'page_url': sr.url,
                            'source': parser.name
                          }})
            except:
                log_exc("Save pdf data error")
            postprocess(data, ctx, pid)
            progress_dict.pop(pid, None)
            return
    progress_dict.pop(pid, None)

def handle_title_query(query):
    query = title_beautify(query)
    log_info("Get title query: {0}".format(query))

     #starts search
    res = search_startswith(query) # and the idf is large
    if res:
        log_info("Found {0} results in db: {1}".format(
            len(res), str([x['_id'] for x in res])))
        return res
    # similar search
    res = similar_search(query)
    if res:
        log_info(u"Found similar results in db: {0}".format(res['_id']))
        return [res]

    # search on web
    searchers = searcher.register_searcher.get_searcher_list()
    parsers = fetcher.register_parser.get_parser_list()
    ctx = JobContext(query)

    args = zip(searchers, [ctx] * len(searchers))
    pool = Pool()
    async_results = [pool.apply_async(searcher_run, arg) for arg in args]

    # Search and get all the results item
    all_search_results = []
    for s in async_results:
        s = s.get(ukconfig.PYTHON_POOL_TIMEOUT)
        if s is None:
            continue
        srs = s['results']

        # try search database with updated title
        try:
            updated_title = s['ctx_update']['title']
        except KeyError:
            pass
        else:
            if updated_title != query:
                query = updated_title
                res = search_exact(query)
                if res:
                    log_info("Found {0} results in db: {1}".format(
                        len(res), str([x['_id'] for x in res])))
                    return res
        all_search_results.extend(srs)

        meta = s.get('ctx_update')
        if meta:
            log_info('Meat update from searcher: {0}'.format(str(meta.keys())))
            ctx.update_meta_dict(meta)
    pool.close()
    pool.terminate()

    # Analyse each result and try to parse info
    download_candidates = []
    parser_used = set()
    found = False
    for sr in all_search_results:
        for parser in parsers:
            if parser.can_handle(sr):
                download_candidates.append((parser, sr))
                if ctx.need_field(parser.support_meta_field):
                    # Already tried this fetcher
                    if not parser.repeatable and \
                            parser.name in parser_used:
                        continue
                    else:
                        parser_used.add(parser.name)

                    succ = parser.fetch_info(ctx, sr)
                    if not succ:
                        continue
                    found = True
                    if ctx.existing is not None:
                        log_info("Found {0} results in db".format(len(ctx.existing)))
                        return [ctx.existing]

    # no metadata or downloadable source found
    if not found and len(download_candidates) == 0:
        return None
    # Save data, return data and start downloading
    try:
        pid = new_paper(ctx)
        ret = [{'_id': pid,
                'title': ctx.title,
                'view_cnt': 1,
                'download_cnt': 0
               }]
        ret[0].update(ctx.meta)

        progress_dict[pid] = 0.0
        if len(download_candidates) > 0:
            thread = Thread(target=start_download, args=(download_candidates,
                                                         ctx, pid))
            thread.start()
        return ret
    except:
        log_exc("Failed to save to db")

sp_searcher = SoPaperSearcher()

def handl_author_query(q):
    db = get_mongo('paper')
    res = list(db.find({'author': q}, SEARCH_RETURN_FIELDS))
    return res

def handle_content_query(query):
    log_info("Get content query: {0}".format(query))
    res = sp_searcher.search(query)
    db = get_mongo('paper')

    def transform(r):
        pid = long(r['_id'])
        # XXX should find use '$in' and then do sorting
        doc = db.find_one({'_id': pid}, SEARCH_RETURN_FIELDS)
        if not doc:
            raise Exception("Impossible! Mongo doesn't have this paper in index: {0}".format(pid))
        doc['content'] = r['content']
        doc['weight'] = r['weight']
        return doc

    ret = map(transform, res)
    return ret

if __name__ == '__main__':
    import sys
    if len(sys.argv) > 1:
        res = handle_title_query(sys.argv[1])
        sys.exit(0)
    #res = handle_title_query('test test test this is not a paper name')
    #res = handle_title_query('Intriguing properties of neural networks')
    res = handle_content_query('neural networks')
    #res = handle_title_query("The WEka data mining software an update")
    #res = handle_title_query("linear")
    #print res




#!../../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: mailutil.py
# Date: 五 6月 13 18:07:29 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from textutil import title_beautify
from ukconfig import MAILSERVER_HOST, MAILSERVER_PORT

import json
import requests
import urllib


PAT = "https://sopaper.net9.org/download?pid={0}"

def sendmail(addr, author, res):
    subject = "[SoPaper] Your Paper Request Has Been Processed"
    content = "Dear {0}, <br/>".format(addr)
    content += "Thanks for using <a href=\"https://sopaper.net9.org\">SoPaper</a>. You have recently requested for papers of {0}. ".format(title_beautify(author))
    content += "Here are the results:<br/><br/>"
    for idx, (pid, title) in enumerate(res):
        content += "<a href=\"{1}\">{2}. {0}</a><br/>\n".format(title_beautify(title), PAT.format(pid), idx)

    content += "<br/>Thanks for your support!<br/>"

    dic = {'addr': addr, 'subject': subject, 'content': content}
    headers = {'Content-type': 'application/json', 'Accept': '*/*'}
    resp = requests.post("http://{0}:{1}/mail".format(MAILSERVER_HOST, MAILSERVER_PORT),
                 data=json.dumps(dic), headers=headers)
    print resp.content


if __name__ == '__main__':
    sendmail('ppwwyyxxc@gmail.com', 'Yuxin Wu')



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: exc.py
# Date: Wed Jul 08 22:52:49 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

class RecoverableErr(Exception):
    pass

class FileCorrupted(RecoverableErr):
    pass



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: pdfprocess.py
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import tempfile
import os
from bson.binary import Binary

from uklogger import *
from ukdbconn import get_mongo
from lib.pdf2html import PDF2Html
from lib.textutil import parse_file_size
from lib.pdfutil import *
import contentsearch

def do_addhtml(data, pid):
    # convert to html
    converter = PDF2Html(data, filename=None)
    npage = converter.get_npages()
    htmls = [Binary(converter.get(x)) for x in range(npage + 1)]
    converter.clean()

    db = get_mongo('paper')
    db.update({'_id': pid}, {'$set': {'page': npage, 'html': htmls}})
    log_info("Add html for pdf {0}, page={1}".format(pid, npage))

def do_compress(data, pid):
    """ this *must* succeed adding the pdf"""
    try:
        # compress
        data = pdf_compress(data)
    except:
        pass

    db = get_mongo('paper')
    db.update({'_id': pid}, {'$set': {'pdf': Binary(data)}} )
    log_info("Updated pdf {0}: size={1}".format(
        pid, parse_file_size(len(data))))
    return data

def do_buildindex(ctx, data, pid):
    text = pdf2text(data)
    db = get_mongo('paper')
    db.update({'_id': pid}, {'$set': {'text': text}})

    doc = {'text': text,
           'title': ctx.title,
           'id': pid
          }

    citedby = ctx.meta.get('citedby')
    if citedby:
        citecnt = len(citedby)
        doc['citecnt'] = citecnt
    if ctx.meta.get('citecnt'):
        citecnt = ctx.meta.get('citecnt')
        doc['citecnt'] = citecnt

    author = ctx.meta.get('author')
    if author:
        doc['author'] = author
    contentsearch.do_add_paper(doc)

def postprocess(data, ctx, pid):
    """ post-process routine right after adding a new pdf"""
    log_info("Start compressing {0}".format(pid))
    data = do_compress(data, pid)

    try:
        log_info("Start converting to html {0}".format(pid))
        do_addhtml(data, pid)
    except Exception as e:
        log_exc("Error converting to html")

    try:
        log_info("Start converting to text {0}".format(pid))
        do_buildindex(ctx, data, pid)
    except Exception as e:
        log_exc("Error converting to text")

if __name__ == '__main__':
    import sys
    f = sys.argv[1]
    data = open(f).read()

    text = pdf2text(data)
    print text



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: direct.py
# Date: 一 6月 09 14:29:17 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from . import register_parser
from .base import FetcherBase, direct_download

@register_parser(name='direct link', urlmatch='.*\.pdf',
                 typematch='directpdf', repeatable=True,
                 priority=10)
class DirectPdf(FetcherBase):
    def _do_download(self, updater):
        return direct_download(self.url, updater)

    def _do_get_title(self):
       raise Exception("Cannot get title from direct pdf link" )

    def _do_get_meta(self):
       return {}



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: google.py
# Date: Wed Mar 11 09:21:06 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from . import register_searcher
from ..job import SearchResult
from ..uklogger import *
from ..lib.textutil import title_correct, filter_title_fileformat
from ..ukconfig import BS_PARSER

import re
import urllib
from bs4 import BeautifulSoup
from urlparse import urlparse
import traceback
import requests

GOOGLE_URL = "https://www.google.com/search?q={0}"

def parse_google_link(url):
    return url      # now it seems to be ok
    real = re.findall('http[^&]*&', url)[0]
    ret = urllib.unquote(real[:-1])
    return ret

@register_searcher(name='Google')
def search(ctx):
    query = ctx.query.lower()

    ret = {}
    ret['ctx_update'] = {}
    srs = []

    headers = { 'Hostname': 'www.google.com',
                'User-Agent': ukconfig.USER_AGENT,
                'Accept-Encoding': 'gzip'
              }
    r = requests.get(GOOGLE_URL.format(query), headers=headers, verify=True)
    text = r.text.encode('utf-8')
    #with open('/tmp/a.html', 'r') as f:
        ##f.write(text)
        #text = f.read()

    def find_citecnt(dom):
        try:
            find = dom.findAll(attrs={'class': 'f slp'})[0]
            find = find.findAll('a')[0].text
            citecnt = re.search('[0-9]+', find).group()
            return int(citecnt)
        except:
            return None

    soup = BeautifulSoup(text, BS_PARSER)
    results = soup.findAll(attrs={'class': 'g'})
    for rst in results:
        try:
            h3 = rst.findAll('h3')
            if not h3:  # frame search, e.g. picture/video/kg
                continue
            real_title = h3[0].get_text()
            tc = title_correct(query, real_title)
            if not tc[0]:
                continue
            # TODO do some title update?
            cnt = find_citecnt(rst)
            if cnt is not None:
                ret['ctx_update']['citecnt'] = cnt
            #findpdf = rst.findAll(attrs={'class': 'mime'})
            findpdf = rst.findAll('span')
            if findpdf and findpdf[0].text == '[PDF]':
                pdflink = rst.findAll('a')[0].get('href')
                try:
                    url = parse_google_link(pdflink)
                except:
                    continue
                srs.append(SearchResult('directpdf', url))
            else:
                url = rst.findAll('a')[0].get('href')
                try:
                    url = parse_google_link(url)
                except:
                    continue
                srs.append(SearchResult(None, url))
        except Exception as e:
            log_exc("Search Item parse error: {0}".format(str(e)))
    ret['results'] = srs
    return ret



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: gscholar.py
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>


from . import register_searcher
from ..job import SearchResult
from ..uklogger import *
from ..lib.textutil import title_correct, filter_title_fileformat, title_beautify
from ..lib.ukutil import ensure_unicode, ensure_bin_str
from ..ukconfig import BS_PARSER

import re
import requests
from bs4 import BeautifulSoup
import urllib
from urlparse import urlparse
import traceback

GOOGLE_SCHOLAR_URL = "https://scholar.google.com/scholar?hl=en&q={0}&btnG=&as_sdt=1%2C5&as_sdtp="

@register_searcher(name='Google Scholar', priority=10)
def search(ctx):
    query = ctx.query.lower()

    ret = {}
    ret['ctx_update'] = {}
    srs = []

    r = requests.get(GOOGLE_SCHOLAR_URL.format(query))
    text = r.text.encode('utf-8')
    #with open('/tmp/b.html', 'r') as f:
        #text = f.read()

    def find_citecnt(dom):
        try:
            find = dom.findAll(attrs={'class': 'gs_ri'})[0]
            find = find.findAll(attrs={'class': 'gs_fl'})[0]
            find = find.findAll('a')[0].text
            cnt = re.search('[0-9]+', find).group()
            return int(cnt)
        except:
            return None


    soup = BeautifulSoup(text, BS_PARSER)
    results = soup.findAll(attrs={'class': 'gs_r'})
    title_updated = None
    for rst in results:
        try:
            h3 = rst.findAll('h3')[0]
            real_title = h3.get_text()
            real_title = filter_title_fileformat(real_title)
            tc = title_correct(query, real_title)
            if not tc[0]:
                continue
            if not title_updated and tc[1]:
                title_updated = ensure_unicode(title_beautify(real_title))
                while True:     # fix things like '[citation][c] Title'
                    new_title = re.sub('^\[[^\]]*\]', '', title_updated).strip()
                    if new_title == title_updated:
                        title_updated = new_title
                        break
                    title_updated = new_title
                log_info(u"Title updated: {0}".format(title_updated))
                ret['ctx_update']['title'] = title_updated

            cnt = find_citecnt(rst)
            if cnt is not None:
                ret['ctx_update']['citecnt'] = cnt

            try:
                url = str(h3.find('a').get('href'))
                srs.append(SearchResult(None, url))
            except:
                pass

            findpdf = rst.findAll(attrs={'class': 'gs_ggs'})
            if findpdf:
                pdflink = findpdf[0].find('a').get('href')
                url = str(pdflink)
                srs.append(SearchResult('directpdf', url))
        except Exception as e:
            log_exc("Search Item parse error: {0}".format(str(e)))
    ret['results'] = srs
    return ret



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: indexer.py
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

__all__ = ['xapian_indexer']

import os
import threading
import shutil
import xappy
from xappy import IndexerConnection, FieldActions, UnprocessedDocument, Field
from xappy import errors
from lib.ukutil import ensure_unicode_anytype as ensure_unicode

from xpcommon import FIELD_NUM, STOPWORDS

class XapianIndexer(object):

    def __init__(self, dirname):
        self.dbPath = os.path.abspath(dirname)

        self.dbconn = IndexerConnection(self.dbPath)

        self.dbconn.add_field_action('title', FieldActions.INDEX_FREETEXT,
                                     weight=5, language='en')
        self.dbconn.add_field_action('text', FieldActions.INDEX_FREETEXT,
                                     language='en', spell=True, stop=STOPWORDS)
        #self.dbconn.add_field_action('citecnt', FieldActions.FACET, type='float')
        #self.dbconn.add_field_action('citecnt', FieldActions.WEIGHT)

        self.lock = threading.Lock()

        for k in FIELD_NUM.keys():
            self.dbconn.add_field_action(k, FieldActions.STORE_CONTENT)

    def add_doc(self, doc):
        """ doc: a dict """
        content = doc['text']
        document = UnprocessedDocument()
        document.fields.append(Field('text', content))

        for k, v in doc.iteritems():
            if k in ['text', 'id']:
                continue
            if type(v) == list:
                for item in v:
                    document.fields.append(Field(k, ensure_unicode(item)))
            else:
                document.fields.append(Field(k, ensure_unicode(v)))
        document.id = str(doc['id'])
        try:
            self.lock.acquire()
            self.dbconn.add(document)
        except errors.IndexerError as e:
            print str(e)
        finally:
            self.lock.release()


    def flush(self):
        self.dbconn.flush()

    def close(self):
        self.dbconn.close()

    def clear(self):
        self.close()
        shutil.rmtree(self.dbPath)
        self.__init__(self.dbPath)



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: job.py
# Date: Thu Jun 18 23:11:07 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from lib.textutil import title_beautify
from lib.ukutil import ensure_unicode
from uklogger import *

class JobContext(object):
    def __init__(self, query):
        query = title_beautify(query)
        self.query = query
        self.success = False
        self.title = query
        self.existing = None
        self.meta = {}

    def update_meta_dict(self, meta):
        if 'title' in meta:
            del meta['title']
        if 'citecnt' not in self.meta and 'citedby' in meta:
            self.meta['citecnt'] = len(meta['citedby'])
        self.meta.update(meta)

    def need_field(self, fields):
        for f in fields:
            if f not in self.meta:
                return True
        return False

    def update_new_title(self, title):
        if title != self.title:
            log_info("Using new title: {0}".format(ensure_unicode(title)))
            self.title = title
            return True
        return False

    def try_update_title_from_search_result(self, s):
        try:
            updated_title = s['ctx_update']['title']
        except KeyError:
            pass
        else:
            self.update_new_title(updated_title)

    def __str__(self):
        d = {'title': self.title,
             'success': self.success,
             'meta': self.meta.keys()
            }
        return str(d)


class SearchResult(object):
    def __init__(self, type, url):
        self.url = url
        self.type = type
        self.searcher = None

    def __str__(self):
        return str(self.searcher) + "; " + \
                str(self.type) + "; " + \
                self.url



#!../../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: run-xp.py
# Date: Fri May 23 20:19:26 2014 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from indexer import *
from searcher import *

import json
import glob
import os
import sys

db = './xapian-database'

def index():
    indexer = XapianIndexer(db)

    for idx, f in enumerate(glob.glob('./zbigniew-herbert/*.txt')):
        text = open(f).read()

        doc = {'text': text}
        doc['id'] = idx
        doc['title'] = os.path.basename(f)
        doc['author'] = ['Yuxin Wu', 'Angela Doudou']
        indexer.add_doc(doc)
    indexer.flush()

def search(query):
    searcher = XapianSearcher(db)
    ret = searcher.search(query)
    print json.dumps(ret)

if sys.argv[1] == 'index':
    index()
elif sys.argv[1] == 'search':
    search(sys.argv[2])



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: searcher.py
# Date: 二 6月 10 04:20:24 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import os
import re

import xappy
from xappy import SearchConnection

from xpcommon import FIELD_NUM,STOPWORDS

class XapianSearcher(object):

    def __init__(self, dirname):
        self.dbPath = os.path.abspath(dirname)
        self.conn = SearchConnection(self.dbPath)
        # can use 'reopen()' to open the db again

    def reopen(self):
        self.conn.reopen()

    def search(self, query, offset=0, page_size=10, summary_len=300):
        query = self.conn.spell_correct(query)
        words = query.split()
        words = [x for x in words if x not in STOPWORDS]
        query = ' OR '.join(words)
        #query = ' '.join(words)
        q = self.conn.query_field('text', query)

        res = self.conn.search(q, offset * page_size, page_size)

        def transform(r):
            doc = {'_id': r.id,
                    'title': r.data['title'][0],
                    'content': r.summarise('text', maxlen=summary_len),
                    'weight': r.weight
                   }
            return doc

        ret = map(transform, res)
        return ret

    def close(self):
        self.conn.close()



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: downloader.py
# Date: Wed Jul 08 22:54:28 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import sys
if __name__ == '__main__':
    sys.path.append('../')
from ..uklogger import *
from ..lib.textutil import parse_file_size
from ..lib.exc import FileCorrupted
from .. import ukconfig

import urllib
import os
import tempfile
import requests
from urlparse import urlparse

class ProgressPrinter(object):
    def __init__(self):
        self.total = 0
        self.last_done_len = -1

    def finish(self, data):
        sys.stdout.write("\n")
        sys.stdout.flush()
        log_info("Download finished")

    def update(self, done):
        assert self.total != 0
        width = 50
        done_len = int(width * done / self.total)
        if done_len > self.last_done_len:
            sys.stdout.write("\r[{0}>{1}]".format('=' * done_len,
                                                  ' ' * (width - done_len)))
            sys.stdout.flush()
        self.last_done_len = done_len

    def set_total(self, size):
        """size: number of bytes"""
        log_info("File size is {0}".format(parse_file_size(size)))
        self.total = size

def wget_download(url, progress_updater, headers=None):
    log_info("Download with wget on {0} ...".format(url))

    headers = ' '.join(['--header="{0}: {1}"'.format(k, v) for k, v
                        in headers.iteritems()])
    tf = tempfile.NamedTemporaryFile(suffix='.pdf', delete=False)
    tf.close()
    # set timeout and retry number
    cmd = 'wget "{0}" -O "{1}" {2} --timeout=5 -t 3'.format(url, tf.name, headers)
    ret = os.system(cmd)
    if ret:
        if os.path.exists(tf.name):
            os.remove(tf.name)
        raise FileCorrupted("wget failed with return code {}".format(ret))
    else:
        data = open(tf.name).read()
        progress_updater.finish(data)
        os.remove(tf.name)
    return data

def requests_download(url, progress_updater, headers=None):
    resp = requests.get(url, stream=True, headers=headers)
    total_length = resp.headers.get('content-length')
    if total_length is None:
        data = resp.content
        progress_updater.finish(data)
        return data
    else:
        total_length = int(total_length)
        if total_length < ukconfig.FILE_SIZE_MINIMUM:
            raise FileCorrupted("File too small: " + parse_file_size(total_length))
        if total_length > ukconfig.FILE_SIZE_MAXIMUM:
            raise FileCorrupted("File too large: " + parse_file_size(total_length))
        progress_updater.set_total(total_length)
        dl = 0
        ret = ""
        for data in resp.iter_content():
            dl += len(data)
            ret += data
            progress_updater.update(dl)
        progress_updater.finish(data)
        return ret

def direct_download(url, progress_updater, headers=None):
    """ download with methods given by ukconfig.download_method
        return the data
    """
    log_info("Directly Download with URL {0} ...".format(url))

    if headers is None:
        headers = {'Host': urlparse(url).netloc,
                   'User-Agent': ukconfig.USER_AGENT,
                   'Connection': 'Keep-Alive'
                  }

    # for test and cmd tools only
    if ukconfig.download_method == 'wget':
        data = wget_download(url, progress_updater, headers)
    else:
        data = requests_download(url, progress_updater, headers)
    if len(data) < ukconfig.FILE_SIZE_MINIMUM:
        raise FileCorrupted("File too small: " + parse_file_size(len(data)))
    return data

if __name__ == '__main__':
    data = direct_download('http://delivery.acm.org/10.1145/330000/322274/p615-yao.pdf?ip=59.66.132.22&id=322274&acc=ACTIVE%20SERVICE&key=BF85BBA5741FDC6E%2E587F3204F5B62A59%2E4D4702B0C3E38B35%2E4D4702B0C3E38B35&CFID=456185443&CFTOKEN=45860210&__acm__=1399725544_eebbed2ce2719c67c7a3642f2b21d80a')
    print data



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# File: ukutil.py
# Date: Thu Jun 18 22:50:33 2015 +0800
# Author: jiakai <jia.kai66@gmail.com>
#         Yuxin Wu <ppwwyyxxc@gmail.com>

"""common utility functions"""
from importlib import import_module
from pkgutil import walk_packages
from datetime import datetime
from subprocess import Popen, PIPE
import tempfile
import os

from .. import ukconfig
from ..uklogger import *


try:
    import magic
except:
    pass

def ensure_unicode_anytype(s):
    if isinstance(s, basestring):
        return ensure_unicode(s)
    return s

def ensure_unicode(s):
    """assert type of s is basestring and convert s to unicode"""
    assert isinstance(s, basestring), 's should be string' + str(s)
    if isinstance(s, str):
        s = s.decode('utf-8')
    return s

def ensure_bin_str(s):
    """assert type of s is basestring and convert s to byte string"""
    assert isinstance(s, basestring), 's should be string'
    if isinstance(s, unicode):
        s = s.encode('utf-8')
    return s

def import_all_modules(file_path, pkg_name):
    """import all modules recursively in a package
    :param file_path: just pass __file__
    :param pkg_name: just pass __name__
    """
    for _, module_name, _ in walk_packages(
            [os.path.dirname(file_path)], pkg_name + '.'):
        import_module(module_name)

def check_buf_filetype(buf, need_type):
    if ukconfig.USE_MAGIC_LIB:
        s = magic.from_buffer(buf)
    else:
        f = tempfile.NamedTemporaryFile(delete=False)
        f.write(buf)
        f.close()
        s = Popen('file "{0}"'.format(f.name),
                  stdout=PIPE, shell=True).stdout.read()
        os.unlink(f.name)
    if s.find(need_type) != -1:
        return True
    else:
        return False

def check_file_type(fname, need_type):
    s = Popen('file "{0}"'.format(fname), stdout=PIPE, shell=True).stdout.read()
    if s.find(need_type) != -1:
        return True
    return False


if __name__ == '__main__':
    print check_filetype(open("./ukconfig.py").read(), 'PDF')



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: base.py
# Date: Thu Jun 25 16:29:22 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from ..uklogger import *
from ..lib.downloader import direct_download, ProgressPrinter
from ..lib.exc import RecoverableErr
from ..lib.textutil import title_beautify

class FetcherBase(object):

    def __init__(self, search_result):
        self.search_result = search_result
        self.url = self.search_result.url
        self.title = None
        self.meta = None
        self.name = None
        self.data = None

        self.headers = None
        try:
            self._do_pre_parse()
        except Exception as e:
            log_exc("Exception in pre-parse")

    def _do_pre_parse(self):
        """ parse right after getting the url"""

    def _do_download(self, updater):
        """ return data, or raise"""

    def _do_get_meta(self):
        """ return dict"""
        pass

    def _do_get_title(self):
        """ return string, or raise"""
        pass

    def get_meta(self):
        if self.meta is not None:
            return self.meta
        self.meta = self._do_get_meta()
        return self.meta

    def get_title(self):
        if self.title is not None:
            if not self.title:
                return None
            else:
                return self.title
        try:
            self.title = title_beautify(self._do_get_title())
            return self.title
        except:
            self.title = ""
            return None

    def download(self, updater=None):
        """ save self.data"""
        if updater is None:
            updater = ProgressPrinter()
        try:
            self.data = self._do_download(updater)
            return True
        except KeyboardInterrupt:
            raise
        except RecoverableErr as e:
            log_err(str(e))
            return False
        except Exception:
            log_exc("Error while downloading with" \
                    "url '{0}'".format(self.url))
            return False

    def get_data(self):
        if self.data is None:
            raise Exception("Cannot call get_data() before download succeeds")
        return self.data



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: authorfetch.py
# Date: 五 6月 13 18:06:08 2014 +0000
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from collections import defaultdict

from lib.mailutil import sendmail
from ukdbconn import get_mongo

def get_paper_list(name):
    db = get_mongo('paper')
    res = list(db.find({'author': name.lower(), 'pdf': {'$exists': True}},
                       {'title': 1}))
    def transform(r):
        return (r["_id"], r["title"])
    return map(transform, res)

def process_fetch_author(name, email):
    l = get_paper_list(name)
    sendmail(email, name, l)
    print "Mail Sent to {0}".format(email)


if __name__ == '__main__':
    print get_paper_list('jie tang')



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: singleton.py
# Date: Fri May 23 22:12:27 2014 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

class Singleton(type):
    def __init__(cls, name, bases, dict):
        super(Singleton, cls).__init__(name, bases, dict)
        cls.instance = None

    def __call__(cls,*args,**kw):
        if cls.instance is None:
            cls.instance = super(Singleton, cls).__call__(*args, **kw)
        return cls.instance

def dec_singleton(cls):
    instances = {}
    def getinstance():
        if cls not in instances:
            instances[cls] = cls()
        return instances[cls]
    return getinstance



#from distutils.core import setup
from setuptools import setup
kwargs = dict(
  name = 'sopaper',
  version = '0.5',
  description = 'Automatically search and download paper',
  author = 'Yuxin Wu',
  author_email = 'ppwwyyxxc@gmail.com',
  url = 'https://github.com/ppwwyyxx/sopaper',
  keywords = ['Utility'],
  packages = ['sopaper', 'sopaper.fetcher',
              'sopaper.lib', 'sopaper.searcher'],
  entry_points={
      'console_scripts': ['sopaper = sopaper.__main__:main']
  },
  include_package_data=True,
  install_requires=['termcolor', 'requests', 'beautifulsoup4']
)
setup(**kwargs)



#!../manage/exec-in-virtualenv.sh
# -*- coding: UTF-8 -*-
# File: test-fetcher.py
# Date: Thu Jun 18 23:27:53 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

import ukconfig
ukconfig.USE_DB = False
ukconfig.USE_INDEXER = False

from fetcher import register_parser, SearchResult
from job import JobContext
from ukdbconn import new_paper

import sys

if __name__ == '__main__':
    if len(sys.argv) == 2:
        ukconfig.USE_DB = True
    ctx = JobContext("Test Filename")

    parser = register_parser.parser_dict['arxiv.org']
    sr = SearchResult(None, "http://arxiv.org/abs/1312.6680")
    #sr = SearchResult(None, "  http://arxiv.org/abs/1404.3610")

    #parser = register_parser.parser_dict['dl.acm.org']
    #url = "http://dl.acm.org/citation.cfm?id=1859761"  # twitter
    #url = "http://dl.acm.org/citation.cfm?id=996342"    # SIFT # Large Number of cited
    #url = "http://dl.acm.org/citation.cfm?id=2366157"  # big
    #url = "http://dl.acm.org/citation.cfm?id=1656278"  # Weka
    #sr = SearchResult(None, url)

    #parser = register_parser.parser_dict['ieeexplore.ieee.org']
    #sr = SearchResult(None, "http://ieeexplore.ieee.org/stamp/stamp.jsp?arnumber=00726791")
    #sr = SearchResult(None, "http://ieeexplore.ieee.org/xpls/abs_all.jsp?arnumber=4244529")


    #parser = register_parser.parser_dict['sciencedirect.com']
    #url = "http://www.sciencedirect.com/science/article/pii/S1570870513000073"
    #sr = SearchResult(None, url)


    #params = parser.fetch_info(ctx, sr)
    #print params
    data = parser.download(sr)

    print ctx.title
    if ukconfig.USE_DB and ctx.success:
        pid = new_paper(ctx)



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: arxiv.py
# Date: Thu Jun 18 23:32:54 2015 +0800
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from . import register_parser
from .base import FetcherBase, direct_download
from ..uklogger import *
from ..ukconfig import BS_PARSER

import re
from bs4 import BeautifulSoup
import requests

#ARXIV_PAT = re.compile('arxiv\.org/[^/]*/(?P<id>.*)')

@register_parser(name='arxiv.org', urlmatch='arxiv.org',
                meta_field=['author', 'bibtex', 'abstract'],
                priority=7)
class Arxiv(FetcherBase):
    def _do_pre_parse(self):
        if 'pdf' in self.url:   # change /pdf/xxx.xxx to /abs/xxx.xxx
            self.url = self.url.replace('pdf', 'abs')
            if self.url.endswith('.abs'):
                self.url = self.url[:-4]
        text = requests.get(self.url).text.encode('utf-8')
        self.soup = BeautifulSoup(text, BS_PARSER)

    def _do_download(self, updater):
        full_text_div = self.soup.findAll('div', attrs={'class': 'full-text'})[0]
        link = full_text_div.findAll('li')[0]
        partial_link = link.children.next().get('href')
        return direct_download('http://arxiv.org' + partial_link, updater)

        #match = ARXIV_PAT.search(self.url).groupdict()
        #pid = match['id']
        #pdflink = "http://arxiv.org/pdf/{0}.pdf".format(pid)
        #return direct_download(pdflink, updater)

    def _do_get_title(self):
        title = self.soup.findAll(attrs={'name': 'citation_title'})[0]
        title = title.get('content')
        return title

    def _do_get_meta(self):
        meta = {}
        try:
            authors = self.soup.findAll(attrs={'class': 'authors'})[0]
            authors = authors.findAll('a')
            author = [a.text for a in authors]
            meta['author'] = author
        except:
            pass

        try:
            abstract = self.soup.findAll(attrs={'class': 'abstract mathjax'})[0]
            abstract = abstract.text.strip()
            abstract = abstract[abstract.find(':')+1:].strip()
            meta['abstract'] = abstract
        except:
            pass

        try:
            bibtex_url = self.soup.findAll(attrs={'title': 'DBLP bibtex record'})
            bibtex_url = bibtex_url[0].get('href')
            bibtex_text = requests.get(bibtex_url).text.encode('utf-8')
            bibtex_soup = BeautifulSoup(bibtex_text, BS_PARSER)
            pre = bibtex_soup.findAll('pre')[0]
            bibtex = pre.text
            meta['bibtex'] = bibtex
        except:
            pass
        return meta



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
# File: __init__.py
# Author: Yuxin Wu <ppwwyyxxc@gmail.com>

from functools import wraps

from ..lib.ukutil import import_all_modules
from ..uklogger import *
from ..job import JobContext

def searcher_run(searcher, ctx):
    """ a global function to invoke with multiprocessing,
        run a searcher against a JobContext"""
    return searcher.run(ctx)

class register_searcher(object):
    searcher_list = []

    def __init__(self, *args, **kwargs):
        self.name = kwargs.pop('name')
        self.priority = kwargs.pop('priority', 5)

    def __call__(self, func):
        """ func: callable to be invoked, took a JobContext
            func cannot change JobContext
            func return a dict with keys:
                results : list of 'SearchResult'
                ctx_update: dict
        """

        @wraps(func)
        def wrapper(ctx):
            assert isinstance(ctx, JobContext)
            try:
                log_info("Searching '{1}' with searcher: '{0}' ...".
                         format(self.name, ctx.query))
                res = func(ctx)
                for r in res['results']:
                    r.searcher = self.name
                return res
            except KeyboardInterrupt:
                raise
            except Exception as e:
                log_exc("Error in searcher '{0}' with query '{1}': {2}".
                        format(self.name, ctx.query, str(e)))

        self.searcher_list.append(self)
        self.cb = wrapper
        return wrapper

    def run(self, ctx):
        """ run this searcher against the context given"""
        res = self.cb(ctx)
        if res and res['results']:
            log_info("Got the following results from {0}:\n".format(self.name) +
                    "\n".join([str(r) for r in res['results']]))
        return res

    @staticmethod
    def get_searcher_list():
        return sorted(register_searcher.searcher_list, key=lambda x: x.priority, reverse=True)

import_all_modules(__file__, __name__)
